package aa4mm.deployment.netlogo.model;

import java.awt.EventQueue;
import java.lang.reflect.InvocationTargetException;
import java.util.Vector;

import javax.swing.JFrame;

import org.nlogo.agent.Agent;
import org.nlogo.agent.AgentSet;
import org.nlogo.api.CompilerException;
import org.nlogo.lite.InterfaceComponent;

import aa4mm.core.model.GenericModelArtefact;
import aa4mm.core.type.*;
import aa4mm.deployment.netlogo.type.*;

/**
 * This class represent the model-artefact in charge of interfacing the NetLogo
 * simulator in a "single" JFrame allow multiple instances running.
 * 
 * @author Julien Siebert, Vincent Chevrier
 * @since Darjeeling
 */
public abstract class NetlogoModelArtefact extends GenericModelArtefact {

	protected static final int WINDOW_WIDTH = 850;
	protected static final int WINDOW_HEIGHT = 800;
	
	/**
	 * specify how to interprete the modelAccesDescription 
	 * and how to laod model in netlogo
	 */
	private boolean forJWSLaunch = false;
	/**
	 * a path or a string corresponding to the model according to the kind of "launch"
	 * 
	 */
	protected String modelAccesDescription; 
	protected String frameName;

	protected InterfaceComponent comp;

	protected javax.swing.JFrame frame;

	public void execCmd(String cmd) {
		try {
			comp.command(cmd);
		} catch (CompilerException e) {
			// TODO Auto-generated catch block
			e.printStackTrace();
		}
	}
	
	public NetlogoModelArtefact(String modelPath, String frameName, boolean forJWSLaunch) {
		super();
		this.forJWSLaunch = forJWSLaunch;
		this.modelAccesDescription = modelPath;
		this.frameName = frameName;
	}
	public NetlogoModelArtefact(String modelPath, String frameName) {
		super();
		this.modelAccesDescription = modelPath;
		this.frameName = frameName;
	}

	/**
	 * 
	 */
	protected void openModel(){
		if (!forJWSLaunch){
			// classical load
			try {
				EventQueue.invokeAndWait(new Runnable() {

					public void run() {
						frame.setSize(WINDOW_WIDTH, WINDOW_HEIGHT);
						frame.add(comp);
						frame.setTitle(frameName);
						frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
						frame.setVisible(true);
						try {

							comp.open(modelAccesDescription);
						} catch (Exception ex) {
							ex.printStackTrace();
						}
						
					}
				});
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
		else {
			try {
				EventQueue.invokeAndWait(new Runnable() {

					public void run() {
						frame.setSize(WINDOW_WIDTH, WINDOW_HEIGHT);
						frame.add(comp);
						frame.setTitle(frameName);
						frame.setVisible(true);
						try {

						
							comp.openFromSource(frameName, null, modelAccesDescription);

						} catch (Exception ex) {
							ex.printStackTrace();
						}
					}
				});
			} catch (InterruptedException e) {
				e.printStackTrace();
			} catch (InvocationTargetException e) {
				e.printStackTrace();
			}
		}
	}
	protected void createSimulatorInstance() {
		// create a new "light" instance
		frame = new JFrame();
		comp = new InterfaceComponent(frame);

		// open the model file
		openModel();

	}

	@Override
	public SimulData getOutputData() {
		return getNetLogoTurtlesFromModel();
	}

	@Override
	public void init() {

		// create a simulator instance
		createSimulatorInstance();

		// set up the model parameters
		execCmd("setup");
		
		
	}

	@Override
	public void run() {

		// run the go procedure (here our policy is to run only one-by-one
		execCmd("go");

	}

	@Override
	public SimulTime getCurrentTime() {
		SimulTime currentTime = new NetLogoTime();
		try {
			currentTime.setTimeFromModel(comp.report("ticks"));
		} catch (CompilerException e) {
			e.printStackTrace();
			System.exit(666);
		}
		return currentTime;
	}

	// -----------------------------------------------------

	/**
	 * This function returns the output data from the model : the turtle
	 * positions of non hidden turtles
	 * 
	 * @return the list of "non hidden turtles" positions from netlogo
	 */
	protected SimulData getNetLogoTurtlesFromModel() {
		// the sheep positions
		Vector<NetlogoTurtle> turtleList = new Vector<NetlogoTurtle>();

		// the list of agents (the sheep)
		AgentSet listOfNetlogoTurtles;

		try {

			// get the list of turtles from netlogo
			listOfNetlogoTurtles = (AgentSet) comp.report(NetlogoTurtle
					.getBreeds() + " with [not hidden?]");

			// check if the agent set is not empty
			if (!listOfNetlogoTurtles.isEmpty()) {

				// // for each sheep get the position and set the
				// // SheepPosition object
				for (int i = 0; i < listOfNetlogoTurtles.count(); i++) {

					try {
						// get the agent
						Agent agt = listOfNetlogoTurtles.agent(i);

						// get the agent heading, xCor, yCor
						Double id = (Double) agt.getVariable(0);
						Double color = (Double) agt.getVariable(1);
						Double heading = (Double) agt.getVariable(2);
						Double xPos = (Double) agt.getVariable(3);
						Double yPos = (Double) agt.getVariable(4);

						NetlogoTurtle tortuga = new NetlogoTurtle(xPos, yPos,
								id, heading, color);
						turtleList.add(tortuga);
					} catch (NullPointerException e) {

					}
				} // end for

			} else {
				System.err.println("all turtles are dead");
				System.exit(666);
			}

		} catch (CompilerException e) {
			e.printStackTrace();
		}

		return new VectorOfNetlogoTurtles(turtleList);
	}

	@Override
	public SimulData getObservationData() {
		throw new UnsupportedOperationException("Not supported yet.");
	}

	// -----------------------------------------------------

}
